require "to_ascii_latex/version"
require 'to_latex'
require 'nokogiri'

module ToAsciiLatex
  @@smartquotes = false
  @@replace_nbsp = false

  def self.smartquotes
    @@smartquotes
  end
  def self.smartquotes=(v)
    raise "Unexpected value #{v.inspect} for smartquotes" unless [true, false].include?(v)
    @@smartquotes = v
  end
  def self.replace_nbsp
    @@replace_nbsp
  end
  def self.replace_nbsp=(v)
    raise "Unexpected value #{v.inspect} for replace_nbsp" unless [true, false].include?(v)
    @@replace_nbsp = v
  end

  # http://www.fileformat.info/info/unicode/char/fb01/index.htm
  # http://www.w3.org/Math/characters/unicode.xml
  # http://www.johndcook.com/unicode_latex.html

  NBSP = "\u00A0"
  MAPPING = {}
  RE_SPECIALS = %w{. | ( ) [ ] { } \ ^ $ + * ?}

  mapping = Nokogiri::XML(open(File.join(File.dirname(__FILE__), 'unicode.xml')))
  mapping.xpath('//character[@id and @mode and latex]').each{|char|
    id = char['id'].to_s
    next if id =~ /^U[-0-9A-F]+$/ && id =~ /-/
    raise "Unexpected char #{id.inspect}" unless id =~ /^U[0-9A-F]+$/i
    id.gsub!(/^U/, '')
    id.gsub!(/^0/, '') if id.size == 5 && id =~ /^0/

    key = [id.gsub(/^U/i, '').hex].pack('U')
    value = char.at('.//latex').inner_text.strip

    value = "``" if value == '\\textquotedblleft'
    value = "''" if value == '\\textquotedblright'
    value = "`" if value == '\\textasciigrave'
    value = "'" if value == '\\textquotesingle'
    value = ' ' if value == "\\space"

    next if key == value

    value = "\\ensuremath{#{value}}" if char['mode'] == 'math'
    value = "{#{value}}" if value !~ /^\\/ || value !~ /}$/
    MAPPING[key] = value
  }
  RE = /#{MAPPING.keys.collect{|c| RE_SPECIALS.include?(c) ? "\\#{c}" : c}.join('|')}/

  # Add a hook - whenever a class or module calls `extend ToAsciiLatex`,
  # run this code
  def self.extended(base)

    # In the context of the extending module or class 
    # (in this case, it will be ToLatex), do the following
    base.class_eval do

      # Define this new method
      def self.new_escape(s)
        s = s.gsub(NBSP, ' ') if @@replace_nbsp
        x = s.gsub(RE){|c| MAPPING[c]} # .gsub('\backslash{}', '\ensuremath{\backslash}')
        warn "Unicode in  #{x.inspect}" if x.delete("^\u{0000}-\u{007F}") != x

        if @@smartquotes
          i = 0
          x.gsub!('"'){|q| i += 1; (i % 2 == 0) ? "''" : "``"}
        end

        return x
      end

      # Set up aliases. 
      # We're already in the context of the class, but there's no
      # `self.alias`, so we need to call `alias` inside this block
      class << self

        # We can call the original `escape` method with `old_escape`
        alias :old_escape :escape

        # If we call `escape`, now we'll get our `new_escape` method
        alias :escape :new_escape
      end
    end

  end

end

# This will fire off the self.extended hook in ToAsciiLatex
ToLatex.extend ToAsciiLatex
